/*-
 * Copyright (C)2003 @BABOLO http://www.babolo.ru/
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: mife.h,v 1.7 2007/05/28 23:34:35 babolo Exp $
 */

#define MIFE_RTRY    0x0000FF /* R  */
#define MIFE_BUFL    0x000F00 /* RW */
#define MIFE_RECY    0x001000 /* W     */
#define MIFE_SYNC    0x002000 /* W    AIO            */
#define MIFE_INMM    0x010000 /* R               */
#define MIFE_ALLC    0x020000 /* R     free()        */
#define MIFE_MMAP    0x040000 /* R   mmaped                    */
#define MIFE_CLOS    0x080000 /* RW    fd           */
#define MIFE_EOFL    0x100000 /* R               */
                              /* W    EOF     */
#define MIFE_FULL    0x200000 /* R    \0            */
#define MIFE_PIPE    0x400000 /* R         */
#define MIFE_BUFX    0x800000 /* R          */

#define kukMIFE_RTRY(x) ((u_int32_t)(x) & MIFE_RTRY)
#define setMIFE_RTRY(a,x) {(a) &= ~MIFE_RTRY; (a) |= kukMIFE_RTRY(x);}
#define getMIFE_RTRY(a) ((a) & MIFE_RTRY)
#define kukMIFE_BUFL(x) (((u_int32_t)(x) << 8) & MIFE_BUFL)
#define setMIFE_BUFL(a,x) {(a) &= ~MIFE_BUFL; (a) |= kukMIFE_BUFL(x);}
#define getMIFE_BUFL(a) (((a) >> 8) & (MIFE_BUFL >> 8))

typedef struct {
    u_int32_t   flag;    /*                                  */
    int         fd;      /*               */
    ssize_t     mindelta;
    size_t      actlen;  /*              */
    size_t      buflen;  /*                           */
    void       *buffer;  /*                                  */
    off_t       offset;  /*            */
    off_t       st_size; /*   ( )              */
    mode_t      st_mode; /*                              */
    size_t      eomax;   /*                          */
    size_t      eotrilen;/*                    */
    ssize_t     eolp;    /*          */
    ssize_t     eolst;   /*     */
    ssize_t    *eotrix;  /*               */
    const char *eoln;    /*                       */
} mife_descriptor;

/*      64 */
typedef struct {
    u_int32_t   flag;    /*                                  */
    union { size_t        pos;
            struct {
                u_int16_t lo;
                u_int16_t hi;
            } wr;
    } towrite;           /*                */
    off_t       offset;  /*            */
    void       *iocb[1]; /*         */
} mife_aout;

mife_descriptor * mife_opef __P((u_int32_t flag, int fd));
mife_descriptor * mife_open __P((u_int32_t flag, const char *path));
mife_descriptor * mife_opes __P((u_int32_t flag, const char *src, size_t size));
mife_descriptor * mife_opem __P((u_int32_t flag, char *src, size_t size));

ssize_t   mife_eolinit __P((mife_descriptor *mife, const char *eoln));
ssize_t mife_eoldetect __P((mife_descriptor *mife));
ssize_t      mife_read __P((mife_descriptor *mife, ssize_t needlen, off_t offset));
void *        mife_get __P((mife_descriptor *mife, off_t offset));
int         mife_close __P((mife_descriptor *mife));

ssize_t      mife_writ __P((int fd, const void *buf, size_t nbytes));
mife_aout * mife_aopen __P((u_int32_t flag, int fd));
int          mife_aput __P((mife_aout *aso, int src));
int        mife_aclose __P((mife_aout *aso));
